@@ -96,7 +96,7 @@ class CORE_EXPORT QgsFeature
     int fieldIdx = sipCpp->fieldNameIndex( *a0 );
     if ( fieldIdx == -1 )
     {
-      PyErr_SetString( PyExc_KeyError, a0->toAscii() );
+      PyErr_SetString( PyExc_KeyError, a0->toLatin1() );
       sipIsErr = 1;
     }
     else
@@ -131,7 +131,7 @@ class CORE_EXPORT QgsFeature
     int fieldIdx = sipCpp->fieldNameIndex( *a0 );
     if ( fieldIdx == -1 )
     {
-      PyErr_SetString( PyExc_KeyError, a0->toAscii() );
+      PyErr_SetString( PyExc_KeyError, a0->toLatin1() );
       sipIsErr = 1;
     }
     else
@@ -163,7 +163,7 @@ class CORE_EXPORT QgsFeature
     int fieldIdx = sipCpp->fieldNameIndex( *a0 );
     if ( fieldIdx == -1 )
     {
-      PyErr_SetString( PyExc_KeyError, a0->toAscii() );
+      PyErr_SetString( PyExc_KeyError, a0->toLatin1() );
       sipIsErr = 1;
     }
     else
@@ -176,9 +176,9 @@ class CORE_EXPORT QgsFeature
      * \param id feature id
      */
 #ifndef SIP_RUN
-    QgsFeature( QgsFeatureId id = QgsFeatureId() );
+    QgsFeature( QgsFeatureId id = FID_NULL );
 #else
-    QgsFeature( qint64 id = 0 );
+    QgsFeature( qint64 id = FID_NULL );
 #endif
 
     /**
@@ -187,9 +187,9 @@ class CORE_EXPORT QgsFeature
      * \param id feature id
      */
 #ifndef SIP_RUN
-    QgsFeature( const QgsFields &fields, QgsFeatureId id = QgsFeatureId() );
+    QgsFeature( const QgsFields &fields, QgsFeatureId id = FID_NULL );
 #else
-    QgsFeature( const QgsFields &fields, qint64 id = 0 );
+    QgsFeature( const QgsFields &fields, qint64 id = FID_NULL );
 #endif
 
     /**
@@ -251,8 +251,8 @@ class CORE_EXPORT QgsFeature
      * The feature will be valid if it was successful.
      * \param field the index of the field to set
      * \param attr the value of the attribute
-     * \returns false, if the field index does not exist
-     * \note For Python: raises a KeyError exception instead of returning false
+     * \returns FALSE, if the field index does not exist
+     * \note For Python: raises a KeyError exception instead of returning FALSE
      * \note Alternatively in Python: @code feature[field] = attr @endcode
      * \see setAttributes
      */
@@ -318,13 +318,13 @@ class CORE_EXPORT QgsFeature
 
     /**
      * Sets the validity of the feature.
-     * \param validity set to true if feature is valid
+     * \param validity set to TRUE if feature is valid
      * \see isValid
      */
     void setValid( bool validity );
 
     /**
-     * Returns true if the feature has an associated geometry.
+     * Returns TRUE if the feature has an associated geometry.
      * \see geometry()
      * \since QGIS 3.0.
      */
@@ -355,7 +355,8 @@ class CORE_EXPORT QgsFeature
      *   feature.setGeometry( QgsGeometry( geometry ) )
      * \endcode
      *
-     * * Example:
+     * ### Example
+     *
      * \code{.py}
      *   # Sets a feature's geometry to a point geometry
      *   feature.setGeometry( QgsPoint( 210, 41 ) )
@@ -392,9 +393,9 @@ class CORE_EXPORT QgsFeature
     /**
      * Assign a field map with the feature to allow attribute access by attribute name.
      *  \param fields The attribute fields which this feature holds
-     *  \param initAttributes If true, attributes are initialized. Clears any data previously assigned.
-     *                        C++: Defaults to false
-     *                        Python: Defaults to true
+     *  \param initAttributes If TRUE, attributes are initialized. Clears any data previously assigned.
+     *                        C++: Defaults to FALSE
+     *                        Python: Defaults to TRUE
      * \see fields
      * \since QGIS 2.9
      */
@@ -407,13 +408,13 @@ class CORE_EXPORT QgsFeature
     QgsFields fields() const;
 
     /**
-     * Insert a value into attribute. Returns false if attribute name could not be converted to index.
+     * Insert a value into attribute. Returns FALSE if attribute name could not be converted to index.
      *  Field map must be associated using setFields() before this method can be used.
      *  The feature will be valid if it was successful
      *  \param name The name of the field to set
      *  \param value The value to set
-     *  \returns false if attribute name could not be converted to index (C++ only)
-     *  \note For Python: raises a KeyError exception instead of returning false
+     *  \returns FALSE if attribute name could not be converted to index (C++ only)
+     *  \note For Python: raises a KeyError exception instead of returning FALSE
      *  \note Alternatively in Python: @code feature[name] = attr @endcode
      *  \see setFields
      */
@@ -425,7 +426,7 @@ class CORE_EXPORT QgsFeature
     int fieldIdx = sipCpp->fieldNameIndex( *a0 );
     if ( fieldIdx == -1 )
     {
-      PyErr_SetString( PyExc_KeyError, a0->toAscii() );
+      PyErr_SetString( PyExc_KeyError, a0->toLatin1() );
       sipIsErr = 1;
     }
     else
@@ -446,8 +447,8 @@ class CORE_EXPORT QgsFeature
      * Removes an attribute value by field name. Field map must be associated using setFields()
      *  before this method can be used.
      *  \param name The name of the field to delete
-     *  \returns false if attribute name could not be converted to index (C++ only)
-     *  \note For Python: raises a KeyError exception instead of returning false
+     *  \returns FALSE if attribute name could not be converted to index (C++ only)
+     *  \note For Python: raises a KeyError exception instead of returning FALSE
      *  \note Alternatively in Python: @code del feature[name] @endcode
      *  \see setFields
      */
@@ -457,7 +458,7 @@ class CORE_EXPORT QgsFeature
     int fieldIdx = sipCpp->fieldNameIndex( *a0 );
     if ( fieldIdx == -1 )
     {
-      PyErr_SetString( PyExc_KeyError, a0->toAscii() );
+      PyErr_SetString( PyExc_KeyError, a0->toLatin1() );
       sipIsErr = 1;
       sipRes = false;
     }
@@ -486,7 +487,7 @@ class CORE_EXPORT QgsFeature
     int fieldIdx = sipCpp->fieldNameIndex( *a0 );
     if ( fieldIdx == -1 )
     {
-      PyErr_SetString( PyExc_KeyError, a0->toAscii() );
+      PyErr_SetString( PyExc_KeyError, a0->toLatin1() );
       sipIsErr = 1;
     }
     else
@@ -535,6 +536,17 @@ class CORE_EXPORT QgsFeature
      */
     int fieldNameIndex( const QString &fieldName ) const;
 
+    /**
+     * Returns the approximate RAM usage of the feature, in bytes.
+     *
+     * This method takes into account the size of variable elements (strings,
+     * geometry, ...), but the value returned should be considered as a lower
+     * bound estimation.
+     *
+     * \since QGIS 3.16
+     */
+    int approximateMemoryUsage() const;
+
     //! Allows direct construction of QVariants from features.
     operator QVariant() const
     {
